/* foo_to_v5d.c */

/* Vis5D version 4.2 */

/*
 * A skeleton of a program for converting your data format to the
 * v5d format.  This version only allows the cylindrical equidistant
 * map projection and linear, equally-spaced vertical coordinate system.
 * Use foo2_to_v5d.c for other map projections.
 *
 *
 * Instructions:
 *   1. You'll probably want to rename this file to something more
 *      descriptive!
 *   2. Read this file from top to bottom.
 *   3. You must know how to read the file format of your data either
 *      using C's standard I/O functions or using a library you may
 *      have for your format (we assume the former case here).
 *   4. You have to provide code to read your file's header to initialize
 *      some variables, and code to read 3-D grids from your file.  Follow
 *      the steps outlined in the program below.
 *   5. The v5d* functions do a lot of error checking to help you.
 *   6. Rename the makefile named foo_to_v5d.c.m to something which
 *      corresponds to the name in (1).
 *   7. Edit the makefile to assign PROGRAM to the name you selected in (1).
 *      Also, check the CFLAGS variable.
 */




#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "binio.h"
#include "v5d.h"
#include "mrcP.h"
/* If you're using a library to read your file, add the #include file here */




/*
 * This is the main conversion function.  The two filename arguments are
 * obtained from the command line.
 * Input:  infile - name of input file in your format
 *         outfile - name of output file in v5d format
 */
static int convert( int argc, char *argv[])
{
   float *g;
   FILE *f;
   int it, iv, ir, ic, il;
   int sz;
int i;
MRC_header hdr;


   /** STEP 1:  The following variables must be initialized in STEP 2.  See
    ** the README file section describing the 'v5dCreateSimple' call for more
    ** information.
    **/
   int NumTimes;			/* number of time steps */
   int NumVars;				/* number of variables */
   int Nr, Nc, Nl;			/* size of 3-D grids */
   char VarName[MAXVARS][10];		/* names of variables */
   int TimeStamp[MAXTIMES];		/* real times for each time step */
   int DateStamp[MAXTIMES];		/* real dates for each time step */
   float NorthLat;			/* latitude of north bound of box */
   float LatInc;			/* spacing between rows in degrees */
   float WestLon;			/* longitude of west bound of box */
   float LonInc;			/* spacing between columns in degs */
   float BottomHgt;			/* height of bottom of box in km */
   float HgtInc;			/* spacing between grid levels in km */


   /**
    ** STEP 2:  Open your file and read the header information to initialize
    ** the above variables.
    **/

   /* example using stdio: */
   /* use fscanf(), fgets(), etc. to read your header if in ASCII, else use */
   /* fread() to read binary data. */

//printf("Image size (x y & z same): ");
//scanf(" %d",&sz);
//if (sz<=1) exit(0);
Nc=Nl=Nr=sz;

f = fopen( argv[1], "r" );
if (!f) {
	printf("Error: couldn't open %s for reading\n", argv[iv+1] );
	exit(1);
}
fread(&hdr,sizeof(MRC_header),1,f);
fclose(f);

Nc=hdr.nx;
Nl=hdr.ny;
Nr=hdr.nz;

NorthLat=10.0;
WestLon=10.0;
BottomHgt=1.0;

LatInc=.01;
LonInc=.01;
HgtInc=.01;

NumTimes=1;
NumVars=argc-2;
TimeStamp[0]=DateStamp[0]=1;
strcpy(VarName[0],"D1");
strcpy(VarName[1],"D2");
strcpy(VarName[2],"D3");
strcpy(VarName[3],"D4");
strcpy(VarName[4],"D5");
strcpy(VarName[5],"D6");


   /** END STEP 2 **/

   /* use the v5dCreateSimple call to create the v5d file and write header */
   if (!v5dCreateSimple(argv[argc-1], NumTimes, NumVars, Nr, Nc, Nl,
			 VarName, TimeStamp, DateStamp,
			 NorthLat, LatInc, WestLon, LonInc,
			 BottomHgt, HgtInc )) {
      printf("Error: couldn't create %s\n", argv[argc-1] );
      exit(1);
   }
   /* allocate space for grid data */
   g = (float *) malloc( Nr * Nc * Nl * sizeof(float) );
   if (!g) {
      printf("Error: out of memory\n");
      exit(1);
   }


   for (it=0;it<NumTimes;it++) {

      for (iv=0;iv<NumVars;iv++) {

	 /**
	  ** STEP 3:  Read your 3-D grid data for timestep it and variable
	  ** iv into the array g here.
	  ** To help with 3-D array indexing we've defined a macro G.
	  ** G(0,0,0) is the north-west-bottom corner, G(Nr-1,Nc-1,Nl-1) is
	  ** the south-east-top corner.  If you want a value to be considered
	  ** missing, assign it equal to the constant MISSING.  For example:
	  ** G(ir,ic,il) = MISSING;
	  **/
#define G(ROW, COLUMN, LEVEL)   g[ (ROW) + ((COLUMN) + (LEVEL) * Nc) * Nr ]

   	f = fopen( argv[iv+1], "r" );
   	if (!f) {
		printf("Error: couldn't open %s for reading\n", argv[iv+1] );
		exit(1);
	}

	fread(&hdr,sizeof(MRC_header),1,f);

	fread(g,sizeof(float)*Nc,Nl*Nr,f);
//	for (i=0; i<Nc*Nl*Nr; i++) g[i]*=100.0;

   	fclose(f);


	 /** END STEP 3 **/

	 /* Write data to v5d file. */
	 if (!v5dWrite( it+1, iv+1, g )) {
	    printf("Error while writing grid.  Disk full?\n");
	    exit(1);
	 }
      }

   }

   v5dClose();
}



/*
 * Main.  You shouldn't have to touch this.
 */
int main( int argc, char *argv[] )
{
   if (argc==1) {
      printf("Usage:\n");
      printf("   %s infile outfile\n", argv[0]);
   }
   else {
      printf("Input file: %s\n", argv[1] );
      printf("Output file: %s\n", argv[2] );
      convert( argc,argv );
   }
   return 0;
}

